<?php
    //  File: MyDebug.php
    /*  This includes PHP code for centralizing the collection and
        reporting all types of errors in PHP code.  */
        
    define( "MYDEBUG_INTERNAL",       0 );
    define( "MYDEBUG_ERRCALLBACK",    1 );
    define( "MYDEBUG_ASSERTCALLBACK", 2 );
    
    define( "MYDEBUG_DISPLAYFILE",    1 );
    define( "MYDEBUG_DISPLAYEMAIL",   2 );
    define( "MYDEBUG_DISPLAYIP",      4 );
    
    function GetPHPErrType( $aErrorNo )
    {
        switch ( $aErrorNo )
        {
            case E_ERROR:
                $aErrorType = "E_ERROR"; // shouldn't occur
                break;
            case E_WARNING:
                $aErrorType = "E_WARNING";
                break;
            case E_PARSE:
                $aErrorType = "E_PARSE"; // shouldn't occur
                break;
            case E_NOTICE:
                $aErrorType = "E_NOTICE";
                break;
            case E_CORE_ERROR:
                $aErrorType = "E_CORE_ERROR"; // shouldn't occur
                break;
            case E_CORE_WARNING:
                $aErrorType = "E_CORE_WARNING"; // shouldn't occur
                break;
            case E_COMPILE_ERROR:
                $aErrorType = "E_COMPILE_ERROR"; // shouldn't occur
                break;
            case E_COMPILE_WARNING:
                $aErrorType = "E_COMPILE_WARNING"; // shouldn't occur
                break;
            case E_USER_ERROR:
                $aErrorType = "E_USER_ERROR";
                break;
            case E_USER_WARNING:
                $aErrorType = "E_USER_WARNING";
                break;
            case E_USER_NOTICE:
                $aErrorType = "E_USER_NOTICE";
                break;
            default:
                $aErrorType = "UNKNOWN";
                break;
        }
        
        return $aErrorType;
    }
    
    // ParseConfig parses the configuration string and sets up
    // the error handler variables
    function ParseConfig( $aConfigStr )
    {
        global $aMyDebugType, $aMyDebugFile, $aMyDebugEmail;
        global $aMyDebugIP, $aMyDebugPort;

        $aConfigArray = explode( ";", $aConfigStr );
        foreach( $aConfigArray as $aItem )
        {
            if ( trim( $aItem != "" ) )
            {
                $aItemArray = explode( "=", $aItem );
                assert( 'count( $aItemArray ) == 2' );
                if ( $aItemArray[1] != "0" )
                {
                    switch ( $aItemArray[0] ) 
                    {
                        case "FILE":
                            $aMyDebugType |= MYDEBUG_DISPLAYFILE;
                            $aMyDebugFile = $aItemArray[1];
                            break;
                        case "MAIL":
                            $aMyDebugType |= MYDEBUG_DISPLAYEMAIL;
                            $aMyDebugEmail = $aItemArray[1];
                            break;
                        case "IP":
                            $aMyDebugType |= MYDEBUG_DISPLAYIP;
                            $aIPArray = explode( ":", $aItemArray[1] );
                            assert( 'count( $aIPArray ) == 2' );
                            $aMyDebugIP = $aIPArray[0];
                            $aMyDebugPort = $aIPArray[1];
                            break;
                    }
                }
            }
        }
    }                   
    
    function CheckFileSanity( $aFile )
    {
        $aFD = fopen( $aFile, "a" );
        if ( $aFD )
        {
            fclose( $aFD );
            return True;
        }
        return False;
    }
    
    function MyDebugShutdown( )
    {
        global $aFileHandle, $aSocketHandle, $aMyDebugType;
        if ( $aMyDebugType & MYDEBUG_DISPLAYFILE ) 
        {
            fclose( $aFileHandle );
        }
    
        if ( $aMyDebugType & MYDEBUG_DISPLAYIP ) 
        {
            fclose( $aSocketHandle );
        }
    }

    /*  This function formats the message type based on where
        it is going to be displayed */
    function FormatType( $aCallType, $aDisplayType )
    {
        switch( $aDisplayType )
        {
            case MYDEBUG_DISPLAYFILE:
            case MYDEBUG_DISPLAYEMAIL:
                switch ( $aCallType )
                {
                    case MYDEBUG_INTERNAL:
                        return "INTERNAL";
                        break;
                    case MYDEBUG_ERRCALLBACK:
                        return "ERROR CALLBACK";
                        break;
                    case MYDEBUG_ASSERTCALLBACK:
                        return "ASSERT CALLBACK";
                        break;
                }
                break;
            case MYDEBUG_DISPLAYIP:
                return $aCallType;
                break;
        }
    }
    
    /*  This function formats the message based on where
        it is going to be displayed and where it came from  */
    function FormatMsg( $aCallType, $aMessage, $aErrType, $aDisplayType )
    {
        $aPrepend = "";
        if ( $aCallType == MYDEBUG_ERRCALLBACK )
        {
            $aPrepend = "PHP error type: " . GetPHPErrType( $aErrType ) . " - ";
        }

        return $aPrepend . $aMessage;
    }
    
    /*  This function formats the message based on where
        it is going to be displayed.  This function relies
        on the FormatContextR recursive function. */
    function FormatContext( $aErrContext, $aDisplayType )
    {
        // for now all display types receive the same context string
        $aString = "";
        $aDelim  = "\n";
        FormatContextR( $aErrContext, $aString, $aDelim );
        return $aString;
    }
    
    function FormatContextR( $aErrContext, &$aString, $aDelim )
    {
        foreach( $aErrContext as $aVarName => $aVarValue )
        {
            if ( is_array( $aVarValue ) == True ) 
            {
                $aString .= "$aVarName = array( ";
                FormatContextR( $aVarValue, $aString, "," );
                $aString .= " )$aDelim";
            }
            else
            {
                $aString .= "$aVarName = {$aVarValue}{$aDelim}";
            }
        }
    }
    
    //  The MyDebug function is the main handler
    function MyDebug( $aMessage, $aFile, $aLine, 
                      $aCallType = MY_DEBUG_INTERNAL, $aErrType = 0,
                      $aErrContext = array() )
    {
        global $aMyDebugType;
        
        for ( $aDisplayType = MYDEBUG_DISPLAYFILE;
              $aDisplayType <= MYDEBUG_DISPLAYIP;
              $aDisplayType++ )
        {
            if ( $aDisplayType & $aMyDebugType )
            {
                $aType      = FormatType( $aCallType, $aDisplayType );
                $aMessage   = FormatMsg ( $aCallType, $aMessage, $aErrType, $aDisplayType );
                
                if ( $aCallType == MYDEBUG_ERRCALLBACK )
                {
                    $aContext = FormatContext( $aErrContext, $aDisplayType );
                }
                else
                {
                    $aContext = "";
                }
                
                MyDebugOutput( $aType, $aMessage, $aFile, $aLine, $aContext, $aDisplayType );
            }
        }
    }   
    
    function MyDebugOutput( $aType, $aMessage, $aFile, $aLine, $aContext, $aDisplayType )
    {
        global $aFileHandle, $aSocketHandle, $aMyDebugEmail;
        
        switch( $aDisplayType )
        {
            case MYDEBUG_DISPLAYFILE:
                $aMsg = "$aType: '$aMessage' occurred in $aFile on line $aLine.  ";
                if ( $aContext != "" )
                {
                    $aMsg .= "Context information follows:\n{$aContext}\n";
                }
                else
                {
                    $aMsg .= "\n";
                }
                fputs( $aFileHandle, $aMsg );
                break;
            case MYDEBUG_DISPLAYEMAIL:
                $aMsg = "$aType: '$aMessage' occurred in $aFile on line $aLine.  ";
                if ( $aContext != "" )
                {
                    $aMsg .= "Context information follows:\n{$aContext}\n";
                }
                else
                {
                    $aMsg .= "\n";
                }
                mail( $aMyDebugEmail, "MyDebug Report", $aMsg, "From: mydebug@host.com\r\n" );
                break;
            case MYDEBUG_DISPLAYIP:
                $aMsg = "$aType|$aMessage|$aFile|$aLine|$aContext^^";
                fputs( $aSocketHandle, $aMsg );
                break;
        }
    }
    
    //  The callback for set_error_handler() function
    function MyErrHandler( $aErrorNo, $aErrorStr, $aFile, $aLine, $aContext) 
    {
        MyDebug( $aErrorStr, $aFile, $aLine, MYDEBUG_ERRCALLBACK,
                 $aErrorNo, $aContext );
    }
    
    //  The callback for assert_options() function
    function MyAssertHandler( $aFileName, $aLineNum, $aAssertion )
    {
        MyDebug( "assert( $aAssertion ) failed", $aFileName, 
                 $aLineNum, MYDEBUG_ASSERTCALLBACK );
    }

    /*  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        Set up the MyDebug environment here.  The following global
        variables are defined:
            $aMyDebugType  - where the debug info will be 'displayed'
            $aMyDebugEmail - email address for sending info
            $aMyDebugFile  - file for logging info
            $aMyDebugIP    - ip for sending info
            $aMyDebugPort  - port for ip (above)    
            $aFileHandle   - file handle for info (if needed)
            $aSocketHandle - socket for info (if needed)
        +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    */
    ParseConfig( getenv( "MYDEBUG_CONFIG" ) );
    if ( $aMyDebugType & MYDEBUG_DISPLAYFILE ) 
    {
        // if we can't write to the log file, tell the user
        // then disable file logging
        if ( CheckFileSanity( $aMyDebugFile ) == False )
        {
            error_log( "MyDebug failed to open file $aMyDebugFile", 0 );
            $aMyDebugType &= ~MYDEBUG_DISPLAYFILE;
        }
    }
    
    if ( $aMyDebugType & MYDEBUG_DISPLAYFILE ) 
    {
        $aFileHandle = fopen( $aMyDebugFile, "a" );
    }

    if ( $aMyDebugType & MYDEBUG_DISPLAYIP ) 
    {
        $aSocketHandle = fsockopen( "udp://$aMyDebugIP", $aMyDebugPort );
    }
    
    //  Now that register the error-handling callbacks and cleanup function 
    set_error_handler( "MyErrHandler" );
    assert_options( ASSERT_CALLBACK, "MyAssertHandler" );
    assert_options( ASSERT_WARNING, 0 );
    register_shutdown_function( "MyDebugShutdown" );
?>